from __future__ import absolute_import

import json
try:
    import Queue
except ImportError as e:
    import queue as Queue
import re
from collections import namedtuple
from threading import Thread

import requests

import yatts.nltextwrap as nltextwrap


class TTS(object):
    """Text to speech object

    Args:
      text: Text that will be transformed into audio
      language: The language of text
      translate_to: (optional) The language to translate the text to
        before processing.  Defaults to en.
      buffer_size: (optional) Number of audio segments to buffer.
        Defaults to 10.
      proxy: (optional) Dict mapping protocol to the URL of the proxy.


    Usage:

      >>> from yatts.google import TTS
      >>> tts = TTS(text='Hello, world!', language='en', translate_to='fr')
      >>> [(segment.text, len(segment.audio)) for segment in tts]
      [(u'Bonjour le monde!', 6192)]

    """
    Segment = namedtuple('Segment', ['text', 'audio'])
    _url = 'http://translate.google.com/translate_tts'

    def __init__(self, text, language='en', translate_to=None,
                 buffer_size=10, proxy=None):
        """Initialize TTS"""

        self.language_from = language
        self.language_to = translate_to or self.language_from

        _nltw = nltextwrap.NLTextWrapper(language=self.language_to, width=100,
                                        expand_tabs=False)
        self.proxy = proxy

        if self.language_from != self.language_to:
            translator = Translator(self.language_from, self.language_to,
                                    proxy=self.proxy)

            translated_text = translator.translate(text)
            text = ' '.join(tt.text for tt in translated_text)

        self.payload = {'tl': self.language_to,
                        'client': 't',
                        'ie': 'UTF-8',
                        'prev': 'input'}

        self.wrapped_text = _nltw.wrap(text)

        self._text_audio_queue = Queue.Queue(maxsize=buffer_size)
        self._cached_segments = []
        self._index = 0
        self._stop = False
        self.stopped = False

        worker = Thread(target=self._downloader)
        worker.setDaemon(True)
        worker.start()

    def save(self, file_name):
        if not self._stop:
            for segment in self:
                pass

        with open(file_name, 'wb') as fout:
            for segment in self._cached_segments:
                fout.write(segment.audio)

    def next(self):
        if self._index < len(self) and not self._stop:
            self._index += 1
            segment = self._text_audio_queue.get()
            self._cached_segments.append(segment)
            return segment
        else:
            raise StopIteration()

    def stop(self):
        """Stop the TTS process"""
        self._stop = True
        # Remove one value from the queue to unblock the put in _downloader
        try:
            self._text_audio_queue.get_nowait()
        except Queue.Empty as e:
            pass

    def _downloader(self):
        for idx, s in enumerate(self.wrapped_text):
            if self._stop:
                self.stopped = True
                break

            self.payload['q'] = s
            self.payload['idx'] = idx
            self.payload['textlen'] = len(s)
            self.payload['total'] = len(self)

            try:
                r = requests.get(self._url,
                                 params=self.payload, proxies=self.proxy)
                self._text_audio_queue.put(self.Segment(s, r.content))
            except requests.exceptions.RequestException as e:
                self.stop()
                raise e
            except Exception as e:
                self.stop()
                raise e

    def __len__(self):
        return len(self.wrapped_text)

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()


class Translator(object):
    """Translator object

    Args:
      language_from: The language of the text to be translated
      translate_to: The language to translate to
      proxy: Dict of http proxy settings
    """
    TranslatedText = namedtuple('TranslatedText',
                                ['text', 'original_text'])
    _url = 'https://translate.google.com/translate_a/single'

    def __init__(self, language_from, language_to, proxy=None):
        """Initialize Translator"""

        self.language_from = language_from
        self.language_to = language_to
        self.proxy = proxy or {}

        self.payload = {'client': 't',
                        'sl': self.language_from,
                        'tl': self.language_to,
                        'dt': 't',
                        'ie': 'UTF-8',
                        'oe': 'UTF-8',
                        'source': 'btn'}

    def translate(self, text):
        """Translate text

        Args:
          text: The text to be translated

        Returns:
          A list of Translator.TranslatedText instances
        """

        translated_text = []

        for paragraph in text.strip('\n').split('\n'):
            self.payload['q'] = paragraph
            r = requests.get(self._url,
                             params=self.payload,
                             proxies=self.proxy)
            rv = r.content.decode('utf-8')
            rv = rv.replace('[,', '[null,')
            rv = re.sub(r'(?<=,)\s*,', 'null,', rv)
            j = json.loads(rv)

            translated_text.extend([self.TranslatedText(*line[0:2])
                                    for line in j[0]])

        return translated_text
