from __future__ import absolute_import
from __future__ import print_function

import sys
from functools import partial
from textwrap import TextWrapper

import yatts.languages as languages

try:
    import nltk
except ImportError:
    import yatts._nltk as nltk


class NLTextWrapper(TextWrapper):
    def __init__(self, language, **kwargs):
        self.lang = languages.nltk.get(language, None)
        TextWrapper.__init__(self)
        for key, val in kwargs.items():
            setattr(self, key, val)
        self._wrap_partial = partial(TextWrapper.wrap, self)

    def _tokenize(self, text):
        py3 = sys.version_info >= (3, 0)
        pickle_path = 'tokenizers/punkt/{}{}.pickle'
        pickle_path = pickle_path.format('PY3/' if py3 else '', self.lang)
        tokenizer = nltk.data.load(pickle_path)
        return tokenizer.tokenize(text)

    def wrap(self, text):
        """Wrap text without breaking sentences when possible

        If nltk is not installed, the texted will be wrapped based just
        on length.

        Args:
          text: A string of text to be wrapped.

        Returns:
          A string of wrapped text.

        Usage:
          >>> text = "A short example.  Does it help?  Yes!"
          >>> # Example from TextWrapper in the standard library
          >>> from textwrap import TextWrapper
          >>> tw = TextWrapper(width=25)
          >>> tw.wrap(text)
          ['A short example.  Does it', 'help?  Yes!']
          >>> # Example from NLTextWrapper
          >>> from nltextwrap import NLTextWrapper
          >>> nltw = NLTextWrapper(language='en', width=25)
          >>> text = "A short example.  Does it help?  Yes!"
          >>> nltw.wrap(text)
          ['A short example.', 'Does it help? Yes!']
        """

        tokens = self._tokenize(text)
        flatten = lambda listoflists: sum(listoflists, [])
        wrapped_text = flatten(map(self._wrap_partial, tokens))

        while True:
            for idx, txt in enumerate(wrapped_text[:-1]):
                if len(txt) + len(wrapped_text[idx+1]) < self.width:
                    wrapped_text[idx] = u'{} {}'.format(wrapped_text[idx],
                                                       wrapped_text[idx+1])
                    del wrapped_text[idx+1]
                    break
            else:
                break

        return wrapped_text
